/** ****************************************************************************
 * @file     Stopwatch.h
 * @brief    Stopwatch measure time of execution in microseconds
 * @author   Andrzej Winczura <winczura@elzab.com.pl>
 * @version  1.00
 * @date     09-11-2018
 *
 * @note
 * Copyright ZUK ELZAB S.A. All rights reserved. @n
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ELZAB S.A.
 * The copyright notice above does not evidence any actual or intended
 * publication of such source code.
 *
 * @par History: @n
 * VERSION - DATE           -   AUTHOR                 -   COMMENT @n
 * v.1.00  - 09-11-2018     -   AW                     -   file creation @n
 *
 *******************************************************************************/
#ifndef _STOPWATCH_H_
#define _STOPWATCH_H_

#include <OS/config.h>  //including hardware configuration i.e. CMSIS, STM HAL, etc.

#define SYSTEM_CORE_CLOCK_IN_MHZ     (SystemCoreClock / (1 MHz))  //[MHz]

//STM ARM Cortex-M3, M4 and M7 have a DWT (Data Watchpoint and Trace) unit implemented.
//When implemented and enabled, CYCCNT increments on each cycle of the processor
//clock. When the counter overflows it wraps to zero, transparently.
//See details: https://static.docs.arm.com/ddi0403/e/DDI0403E_B_armv7m_arm.pdf
#if ((__CORTEX_M == 0x03U) || (__CORTEX_M == 0x04U) || (__CORTEX_M == 0x07U))

  /*!< TRCENA: Enable trace and debug block DEMCR (Debug Exception and Monitor Control Register */
  #define ENABLE_DEMCR()              CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;
  /*!< Reset cycle counter */
  #define RESET_CYCLE_COUNTER()       DWT->CYCCNT = 0
  /*!< Enable cycle counter */
  #define ENABLE_CYCLE_COUNTER()      DWT->CTRL |= DWT_CTRL_CYCCNTENA_Msk
  /*!< Disable cycle counter */
  #define DISABLE_CYCLE_COUNTER()     DWT->CTRL &= ~DWT_CTRL_CYCCNTENA_Msk
  /*!< Read cycle counter register */
  #define GET_CYCLE_COUNTER()         DWT->CYCCNT

  #if (__CORTEX_M == 0x07U)
    #define UNLOCK_DWT()              DWT->LAR = 0xC5ACCE55;    //ITM Lock Access Register, C5ACCE55 enables more write access to Control Register 0xE00 :: 0xFFC
  #else
    #define UNLOCK_DWT()
  #endif

#else

  #define TRCENA_BIT
  #define INIT_CYCLE_COUNTER()
  #define RESET_CYCLE_COUNTER()
  #define ENABLE_CYCLE_COUNTER()
  #define DISABLE_CYCLE_COUNTER()
  #define GET_CYCLE_COUNTER()         ((u32)0)

#endif


/** ****************************************************************************
 * @class cStopwatch
 * @brief Get time of execution in [us] base on Cycle Counting
 *
 * @note It modify the DWT_CYCCNT register
 *
 *******************************************************************************/
class cStopwatch
{
public:
  // -------- variables
  // -------- functions
  /// Class constructor
  cStopwatch();

  /// Class destructor
  ~cStopwatch();

  /// Start counting
  void Start();

  /// Get time of execution (from last Start() or last GetTime())
  u32 GetTime();

private:
  /// cycles value
  u32 cycles;

};

#endif //_STOPWATCH_H_


// -----------------------------------------------------------------------------
// End of file -----------------------------------------------------------------
// -----------------------------------------------------------------------------

